@@ -57,12 +57,20 @@ void page_set_scan_area (Page *page, gint width, gint rows, gint dpi);
 
 gint page_get_dpi (Page *page);
 
+gboolean page_is_landscape (Page *page);
+
 gint page_get_width (Page *page);
 
 gint page_get_height (Page *page);
 
+gint page_get_scan_width (Page *page);
+
+gint page_get_scan_height (Page *page);
+
 void page_start (Page *page);
 
+gboolean page_has_data (Page *page);
+
 gint page_get_scan_line (Page *page);
 
 void page_parse_scan_line (Page *page, ScanLine *line);
@@ -77,10 +85,6 @@ void page_rotate_left (Page *page);
 
 void page_rotate_right (Page *page);
 
-gint page_get_width (Page *page);
-
-gint page_get_height (Page *page);
-
 void page_set_no_crop (Page *page);
 
 void page_set_custom_crop (Page *page, gint width, gint height);
