@@ -98,6 +98,8 @@ struct ScannerPrivate
     SANE_Byte *buffer;
 
     SANE_Int bytes_remaining, line_count, pass_number, page_number, notified_page;
+  
+    gboolean scanning;
 };
 
 G_DEFINE_TYPE (Scanner, scanner, G_TYPE_OBJECT);
@@ -111,7 +113,10 @@ static gboolean
 send_signal (SignalInfo *info)
 {
     g_signal_emit (info->instance, signals[info->sig], 0, info->data);
-    
+
+    if (info->sig == DOCUMENT_DONE || info->sig == SCAN_FAILED)
+        info->instance->priv->scanning = FALSE;
+
     switch (info->sig) {
     case UPDATE_DEVICES:
         {
@@ -186,7 +191,7 @@ static gint get_device_weight (const gchar *device)
     /* Use webcams as a last resort */
     if (g_str_has_prefix (device, "vfl:"))
        return 2;
-   
+
     return 1;
 }
 
@@ -361,12 +366,12 @@ do_redetect (Scanner *scanner)
 
 
 static gboolean
-control_option (SANE_Handle handle, SANE_Value_Type type, SANE_Int index, SANE_Action action, void *value)
+control_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int index, SANE_Action action, void *value)
 {
     SANE_Status status;
     
     status = sane_control_option (handle, index, action, value, NULL);
-    switch (type) {
+    switch (option->type) {
     case SANE_TYPE_BOOL:
         g_debug ("sane_control_option (%d, %s, %s) -> %s",
                  index, get_action_string (action),
@@ -394,16 +399,16 @@ control_option (SANE_Handle handle, SANE_Value_Type type, SANE_Int index, SANE_A
     default:
         break;
     }
-  
+
     if (status != SANE_STATUS_GOOD)
-        g_warning ("Error setting control option: %s", sane_strstatus(status));
+        g_warning ("Error setting option %s: %s", option->name, sane_strstatus(status));
 
     return status == SANE_STATUS_GOOD;
 }
 
 
 static void
-set_default_option (SANE_Handle handle, SANE_Int option_index)
+set_default_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index)
 {
     SANE_Status status;
     
@@ -411,7 +416,7 @@ set_default_option (SANE_Handle handle, SANE_Int option_index)
     g_debug ("sane_control_option (%d, SANE_ACTION_SET_AUTO) -> %s",
              option_index, get_status_string (status));
     if (status != SANE_STATUS_GOOD)
-        g_warning ("Error setting control option: %s", sane_strstatus(status));
+        g_warning ("Error setting default option %s: %s", option->name, sane_strstatus(status));
 }
 
 
@@ -420,7 +425,7 @@ set_bool_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_
 {
     SANE_Bool v = value;
     g_return_if_fail (option->type == SANE_TYPE_BOOL);
-    control_option (handle, SANE_TYPE_BOOL, option_index, SANE_ACTION_SET_VALUE, &v);
+    control_option (handle, option, option_index, SANE_ACTION_SET_VALUE, &v);
 }
 
 
@@ -439,18 +444,68 @@ set_int_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_I
         if (v > option->constraint.range->max)
             v = option->constraint.range->max;
     }
-    control_option (handle, SANE_TYPE_INT, option_index, SANE_ACTION_SET_VALUE, &v);
+    else if (option->constraint_type == SANE_CONSTRAINT_WORD_LIST) {
+        int i;
+        SANE_Int min = INT_MIN, max = INT_MAX;
+      
+        /* Find nearest value above and below requested */
+        for (i = 0; i < option->constraint.word_list[0]; i++) {
+            SANE_Int x = option->constraint.word_list[i+1];
+            if (x <= v && x > min)
+                min = x;
+            if (x >= v && x < max)
+                max = x;
+        }
+      
+        /* Pick nearest */
+        if (max - v < v - min)
+            v = max;
+        else
+            v = min;
+    }
+
+    control_option (handle, option, option_index, SANE_ACTION_SET_VALUE, &v);
 }
 
 
 static void
 set_fixed_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, double value)
 {
-    SANE_Fixed v = SANE_FIX (value);
+    SANE_Fixed v;
 
     g_return_if_fail (option->type == SANE_TYPE_FIXED);
 
-    control_option (handle, SANE_TYPE_FIXED, option_index, SANE_ACTION_SET_VALUE, &v);
+    if (option->constraint_type == SANE_CONSTRAINT_RANGE) {
+        double min = SANE_UNFIX (option->constraint.range->min);
+        double max = SANE_UNFIX (option->constraint.range->max);
+
+        if (value < min)
+            value = min;
+        if (value > max)
+            value = max;
+    }
+    else if (option->constraint_type == SANE_CONSTRAINT_WORD_LIST) {
+        int i;
+        double min = DBL_MIN, max = DBL_MAX;
+      
+        /* Find nearest value above and below requested */
+        for (i = 0; i < option->constraint.word_list[0]; i++) {
+            double x = SANE_UNFIX (option->constraint.word_list[i+1]);
+            if (x <= value && x > min)
+                min = x;
+            if (x >= value && x < max)
+                max = x;
+        }
+      
+        /* Pick nearest */
+        if (max - value < value - min)
+            value = max;
+        else
+            value = min;
+    }
+
+    v = SANE_FIX (value);
+    control_option (handle, option, option_index, SANE_ACTION_SET_VALUE, &v);
 }
 
 
@@ -467,7 +522,7 @@ set_string_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SAN
     size = option->size > value_size ? option->size : value_size;
     string = g_malloc(sizeof(char) * size);
     strcpy (string, value);
-    result = control_option (handle, SANE_TYPE_STRING, option_index, SANE_ACTION_SET_VALUE, string);
+    result = control_option (handle, option, option_index, SANE_ACTION_SET_VALUE, string);
     g_free (string);
    
     return result;
@@ -567,16 +622,24 @@ log_option (SANE_Int index, const SANE_Option_Descriptor *option)
 
     switch (option->constraint_type) {
     case SANE_CONSTRAINT_RANGE:
-        g_string_append_printf (string, " min=%d, max=%d, quant=%d",
-                                option->constraint.range->min, option->constraint.range->max,
-                                option->constraint.range->quant);
+        if (option->type == SANE_TYPE_FIXED)
+            g_string_append_printf (string, " min=%f, max=%f, quant=%d",
+                                    SANE_UNFIX (option->constraint.range->min), SANE_UNFIX (option->constraint.range->max),
+                                    option->constraint.range->quant);
+        else
+            g_string_append_printf (string, " min=%d, max=%d, quant=%d",
+                                    option->constraint.range->min, option->constraint.range->max,
+                                    option->constraint.range->quant);
         break;
     case SANE_CONSTRAINT_WORD_LIST:
         g_string_append (string, " values=[");
         for (i = 0; i < option->constraint.word_list[0]; i++) {
             if (i != 0)
                 g_string_append (string, ", ");
-            g_string_append_printf (string, "%d", option->constraint.word_list[i+1]);
+            if (option->type == SANE_TYPE_INT)
+                g_string_append_printf (string, "%d", option->constraint.word_list[i+1]);
+            else
+                g_string_append_printf (string, "%f", SANE_UNFIX (option->constraint.word_list[i+1]));
         }
         g_string_append (string, "]");
         break;
@@ -671,6 +734,9 @@ close_device (Scanner *scanner)
     GList *iter;
 
     if (scanner->priv->handle) {
+        sane_cancel (scanner->priv->handle);
+        g_debug ("sane_cancel ()");
+
         sane_close (scanner->priv->handle);
         g_debug ("sane_close ()");
         scanner->priv->handle = NULL;
@@ -755,7 +821,7 @@ do_open (Scanner *scanner)
     scanner->priv->notified_page = -1;
     scanner->priv->option_index = 0;
 
-    if (!job->device)
+    if (!job->device && scanner->priv->default_device)
         job->device = g_strdup (scanner->priv->default_device->name);
 
     if (!job->device) {
@@ -824,14 +890,28 @@ do_get_option (Scanner *scanner)
     }
 
     log_option (option_index, option);
-    if (!option->name)
+  
+    /* Ignore groups */
+    if (option->type == SANE_TYPE_GROUP)
+        return;
+
+    /* Option disabled */
+    if (option->cap & SANE_CAP_INACTIVE)
+        return;
+  
+    /* Some options are unnammed (e.g. Option 0) */
+    if (option->name == NULL)
         return;
 
     if (strcmp (option->name, SANE_NAME_SCAN_RESOLUTION) == 0) {
-        if (option->type == SANE_TYPE_FIXED)
-            set_fixed_option (scanner->priv->handle, option, option_index, job->dpi);
-        else
-            set_int_option (scanner->priv->handle, option, option_index, job->dpi);
+        if (option->type == SANE_TYPE_FIXED) {
+            double dpi = job->dpi;
+            set_fixed_option (scanner->priv->handle, option, option_index, dpi);
+        }
+        else {
+            SANE_Int dpi = job->dpi;
+            set_int_option (scanner->priv->handle, option, option_index, dpi);
+        }
     }
     else if (strcmp (option->name, SANE_NAME_SCAN_SOURCE) == 0) {
         const char *adf_sources[] =
@@ -839,6 +919,8 @@ do_get_option (Scanner *scanner)
             "Automatic Document Feeder",
             SANE_I18N ("Automatic Document Feeder"),
             "ADF",
+            SANE_I18N ("ADF Front"),
+            "ADF Front",
             NULL
         };
 
@@ -847,7 +929,7 @@ do_get_option (Scanner *scanner)
                  g_warning ("Unable to set ADF source, please file a bug");
         }
         else {
-            set_default_option (scanner->priv->handle, option_index);
+            set_default_option (scanner->priv->handle, option, option_index);
         }
     }
     else if (strcmp (option->name, SANE_NAME_BIT_DEPTH) == 0) {
@@ -905,22 +987,53 @@ do_get_option (Scanner *scanner)
             break;
         }
     }
-          
+    /* Disable compression, we will compress after scanning */
+    else if (strcmp (option->name, "compression") == 0) {
+        const char *disable_compression_names[] =
+        {
+            SANE_I18N ("None"),
+            SANE_I18N ("none"),
+            "None",
+            "none",
+            NULL
+        };
+            
+        if (!set_constrained_string_option (scanner->priv->handle, option, option_index, disable_compression_names))
+            g_warning ("Unable to disable compression, please file a bug");
+    }
+    /* Always use maximum scan area - some scanners default to using partial areas.  This should be patched in sane-backends */
+    else if (strcmp (option->name, SANE_NAME_SCAN_BR_X) == 0 ||
+             strcmp (option->name, SANE_NAME_SCAN_BR_Y) == 0) {
+        switch (option->constraint_type)
+        {
+        case SANE_CONSTRAINT_RANGE:
+            if (option->type == SANE_TYPE_FIXED)
+                set_fixed_option (scanner->priv->handle, option, option_index, SANE_UNFIX (option->constraint.range->max));
+            else
+                set_int_option (scanner->priv->handle, option, option_index, option->constraint.range->max);
+            break;
+        default:
+            break;
+        }
+    }
+
     /* Test scanner options (hoping will not effect other scanners...) */
-    else if (strcmp (option->name, "hand-scanner") == 0) {
-        set_bool_option (scanner->priv->handle, option, option_index, FALSE);
-    }
-    else if (strcmp (option->name, "three-pass") == 0) {
-        set_bool_option (scanner->priv->handle, option, option_index, FALSE);
-    }                    
-    else if (strcmp (option->name, "test-picture") == 0) {
-        set_string_option (scanner->priv->handle, option, option_index, "Color pattern");
-    }
-    else if (strcmp (option->name, "read-delay") == 0) {
-        set_bool_option (scanner->priv->handle, option, option_index, TRUE);
-    }
-    else if (strcmp (option->name, "read-delay-duration") == 0) {
-        set_int_option (scanner->priv->handle, option, option_index, 200000);
+    if (strcmp (scanner->priv->current_device, "test") == 0) {
+        if (strcmp (option->name, "hand-scanner") == 0) {
+            set_bool_option (scanner->priv->handle, option, option_index, FALSE);
+        }
+        else if (strcmp (option->name, "three-pass") == 0) {
+            set_bool_option (scanner->priv->handle, option, option_index, FALSE);
+        }                    
+        else if (strcmp (option->name, "test-picture") == 0) {
+            set_string_option (scanner->priv->handle, option, option_index, "Color pattern");
+        }
+        else if (strcmp (option->name, "read-delay") == 0) {
+            set_bool_option (scanner->priv->handle, option, option_index, TRUE);
+        }
+        else if (strcmp (option->name, "read-delay-duration") == 0) {
+            set_int_option (scanner->priv->handle, option, option_index, 200000);
+        }
     }
 }
 
@@ -1032,6 +1145,9 @@ do_complete_page (Scanner *scanner)
   
     job = (ScanJob *) scanner->priv->job_queue->data;
 
+    sane_cancel (scanner->priv->handle);
+    g_debug ("sane_cancel ()");
+
     /* If multi-pass then scan another page */
     if (!scanner->priv->parameters.last_frame) {
         scanner->priv->pass_number++;
@@ -1219,6 +1335,13 @@ scanner_redetect (Scanner *scanner)
 }
 
 
+gboolean
+scanner_is_scanning (Scanner *scanner)
+{
+    return scanner->priv->scanning;
+}
+
+
 void
 scanner_scan (Scanner *scanner, const char *device,
               gint dpi, ScanMode scan_mode, gint depth, gboolean multi_page)
@@ -1235,6 +1358,7 @@ scanner_scan (Scanner *scanner, const char *device,
     request->job->depth = depth;
     request->job->multi_page = multi_page;
     g_async_queue_push (scanner->priv->scan_queue, request);
+    scanner->priv->scanning = TRUE;
 }
 
 
